@@ -512,8 +512,8 @@ BOOL WINAPI DllMain([[maybe_unused]] HINSTANCE hinst, DWORD dwReason, [[maybe_un
     }
     return TRUE;
 }
-FRAMEWORK_EXPORT PlatformShim *get_platform_shim(std::vector<fs::FolderManager> *folders) {
-    platform_shim = PlatformShim(folders);
+FRAMEWORK_EXPORT PlatformShim *get_platform_shim(GetFoldersFunc get_folders_by_name_function) {
+    platform_shim = PlatformShim(get_folders_by_name_function);
     return &platform_shim;
 }
 }
